#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <glob.h>
#include <errno.h>

void usage()
{
    printf("Usage: wren <in> <out>\n\n");
    printf("in and out are patterns, each of which may contain wildcards\n"
	   "(either '*' or '?').\n");
    exit(1);
}

void wrename(const char * filename, const char * frompat, const char * topat)
{
    char outname[256];
    int	 it, in, iF, io;

    it = 0;
    in = 0;
    iF = 0;
    io = 0;

    while (topat[it])
    {
	if (topat[it] == '*' || topat[it] == '?')
	{
	    while (frompat[iF] != '*' && frompat[iF] != '?')
		iF ++, in ++;

	    if (topat[it] == '*')
	    {
		char stop = frompat[++iF];

		if (frompat[iF] == '?')
		{
		    fprintf(stderr, "source pattern has ?, destination *\n");
		    exit(1);
		}

		/* process a * - basically carry on until we reach the
		   same character as the pattern has next... not perfect,
		   but it'll do as a start. If anyone wants to replace this
		   with an algorithm that works, I'd be grateful! */

		while(filename[in] && filename[in] != stop)
		{
		    outname[io++] = filename[in++];
		}
	    }
	    else
	    {
		if (frompat[iF] == '*')
		{
		    fprintf(stderr, "source pattern has *, destination ?\n");
		    exit(1);
		}
		outname[io++] = filename[in++];
		iF++;
	    }
	    it++;
	}
	else
	    outname[io++] = topat[it++];
    }
    outname[io] = 0;
    printf("%s => %s\n", filename, outname);
    if (rename(filename, outname))
	perror(filename);
}

int glob_errfunc(const char * filename, int errorcode)
{
    fprintf(stderr, "glob failed (%s,%d)\n", filename, errorcode);
    exit(1);
}

int main(int argc, char ** argv)
{
    glob_t	files;
    int		err, i;

    if (argc != 3) usage();

    if ((err = glob(argv[1], GLOB_NOSORT, &glob_errfunc, &files)))
    {
	if (err == GLOB_NOMATCH) {
	    fprintf(stderr, "file not found\n");
	    exit(1);
	}
	fprintf(stderr, "glob failed\n");
	exit(1);
    }

    if (! files.gl_pathc)
    {
	fprintf(stderr, "file not found (why didn't glob return nomatch?)\n");
	exit(1);
    }

    for (i = 0; i < files.gl_pathc; i++) {
	wrename(files.gl_pathv[i], argv[1], argv[2]);
    }

    return 0;
}
